/*
 * Copyright (C) 2004 Clemens Fuchslocher
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "GlUncompileVisitor.h"


namespace nsSceneGraph {

GlUncompileVisitor::~GlUncompileVisitor(void)
{
	DEBUG_MESSAGE("GlUncompileVisitor::~GlUncompileVisitor()");
}


inline void GlUncompileVisitor::action(GraphNode *node)
{
	if (node->isNodeCompiled())
	{
		glDeleteLists(node->getCompiledDisplayList(), 1);
		node->setCompiledStatus(false);
	}
}


void GlUncompileVisitor::visit(Group *group)
{
	DEBUG_MESSAGE("GlUncompileVisitor::visit(Group *group)")

	this->action(group);
}


void GlUncompileVisitor::visit(Material *material)
{
	DEBUG_MESSAGE("GlUncompileVisitor::visit(Material *material)")

	this->action(material);
}


void GlUncompileVisitor::visit(Transform *transform)
{
	DEBUG_MESSAGE("GlUncompileVisitor::visit(Transform *transform)")

	this->action(transform);
}


void GlUncompileVisitor::visit(VertexArray *array)
{
	DEBUG_MESSAGE("GlUncompileVisitor::~visit(VertexArray *array)");

	while (array->normalArray.empty() == false)
	{
		array->normalArray.pop_back();
	}

	while (array->normalIndex.empty() == false)
	{
		array->normalIndex.pop_back();
	}

	this->action(array);
}


}

